/* Simple table-lookup trig routines */

/* Written by Bernie Roehl, November 1994 for VictorMaxx Technologies */

/* Redistribute at will, but charge nothing and give credit where due */

/* Table lookups for trig.  There are *much* better ways of doing this
   (sin_table and cos_table can be the same table, and only one quadrant
   actually needs to be stored); however, the goal here is simplicity
   and clarity. */

/* Angles are stored as integers, in the range -1800 to 1800
   (0.1 degree accuracy, same as the tracker) */

#include "3d.h"

#include <math.h>

static float sin_table[3600], cos_table[3600];

#define degrees_to_radians(angle) ((angle) * 3.14159252 / 180.0)

void BuildTrigTables(void)
	{
	int angle;
	for (angle = -1800; angle < 1800; ++angle)
		{
		sin_table[angle+1800] = sin(degrees_to_radians(angle / 10.0));
		cos_table[angle+1800] = cos(degrees_to_radians(angle / 10.0));
		}
	}

float sine(int angle)
	{
	return sin_table[angle+1800];
	}

float cosine(int angle)
	{
	return cos_table[angle+1800];
	}
