// MaxxCom.cpp -- source file for CyberMaxx Tracker
// Copyright (C) 1995  VictorMaxx Technologies, Inc.
// Author: Robert J. Gebis
// Date: 2 Oct 95 Ver 1.0


#include <stdafx.h>
#include <windows.h>
#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include "MaxxCom.h"
#include "CyberTrak.h "
#include "Cyberdlg.h"


MaxxCom::~MaxxCom() {};
			
MaxxCom::MaxxCom()
{
	SamplingRate = 40;
	iPortSpeed = 19200;	// will be set in autodet mode to 9600
	bPortInUse = FALSE;
	bChecksum = FALSE;
	port_ID = 0;
	numToReadInBinMode = 6;
	numOfBytesToWrite = 1;
	memset( inBuffer, '\0', 30 );
}


BOOL	MaxxCom::OpenCom(const char* com_id)
{
   	hCom = CreateFile( com_id , GENERIC_READ | GENERIC_WRITE,
								 0, NULL, OPEN_EXISTING, 0, NULL );

		if (hCom == INVALID_HANDLE_VALUE)
		   {
		      CloseCom( hCom);			//  ERROR Close ComPort
			  return FALSE;
		   }
		else
		     {	
		     	COMMTIMEOUTS timeouts;
					// Set timeouts so ReadFile does not
					// return until data is available
						timeouts.ReadIntervalTimeout = 5 ; 
						timeouts.ReadTotalTimeoutMultiplier = 5 ;
						timeouts.ReadTotalTimeoutConstant = 5 ;
						timeouts.WriteTotalTimeoutMultiplier = 0 ;
						timeouts.WriteTotalTimeoutConstant = 0 ;
						SetCommTimeouts(hCom, &timeouts ) ;

		     	// Set Comm Port to 9600 bps
			 	set9600();

			   	if( FALSE == checkForCyberMaxx())
				  {
					  CloseCom( hCom);			//  ERROR CyberMaxx not Found
			  		   return FALSE;
				  }
			 }
	// flag port state
	 bPortInUse = TRUE;

 return TRUE;
}	// end of OpenCom



BOOL	MaxxCom::CloseCom(const HANDLE ComPort)
{
	if( TRUE == CloseHandle( ComPort))
	  {
	    bPortInUse = FALSE;
		port_ID = 0;
 		return TRUE;
	  }
	return FALSE;
}	// end of CloseCom


	
void	MaxxCom::CloseCom(void)
{
 if( port_ID == 0)		// No port connected
 	return;
	if( FALSE == CloseHandle( hCom ))
	  	errMsgBox(" Error.. Can't Close port"); 
 
}	// end of CloseCom


BOOL	MaxxCom::AutoDetectComPort(HWND wnd)
{

	if( bPortInUse == TRUE)
  	  {
  	  	errMsgBox("Device busy or not available");
		return FALSE;
  	  }	 
    

			// Try Com1
				 if(TRUE == OpenCom(COM1))
				 	{
					  CheckRadioButton(wnd , IDC_COM1, IDC_COM4, IDC_COM1);
					  port_ID = 1;
					  return TRUE;
					}
			 
			// Try Com2
				 else if(TRUE == OpenCom(COM2))
				 	{
					  CheckRadioButton( wnd , IDC_COM1, IDC_COM4, IDC_COM2);
					  port_ID = 2;
					  return TRUE;
					}

			// Try Com3
				 else if(TRUE == OpenCom(COM3))
				 	{
					  CheckRadioButton( wnd , IDC_COM1, IDC_COM4, IDC_COM3);
					  port_ID = 3;
					  return TRUE;
					}

			// Try Com4
				 else if(TRUE == OpenCom(COM4))
				 	{
					  CheckRadioButton( wnd, IDC_COM1, IDC_COM4, IDC_COM4);
					  port_ID = 4;
					  return TRUE;
					}

			// ERROR.. No CyberMaxx available
				else { errMsgBox("CyberMaxx not available\nCheck your connections"); }

 return FALSE;
}		// end AutoDetectComPort

													
 
BOOL MaxxCom::getDCB()
{
 BOOL	fSuccess;

  fSuccess = GetCommState(hCom, &dcb);

if (!fSuccess) 		  // no success?
   	return FALSE;
else return TRUE; 
}	// end of getDCB

						

BOOL	MaxxCom::checkForCyberMaxx(void)
{
  static	DWORD		numOfBytesToRead(3);
   
  	memset( inBuffer, '\0', 30 );
	  if( TRUE == WriteFile( hCom,REQUEST_HANDSHAKE, numOfBytesToWrite, &numOfBytesRead, NULL)) {
		 if(TRUE == ReadFile(hCom, inBuffer, numOfBytesToRead, &numOfBytesRead ,NULL)) {
			if ( !strcmp((char*)inBuffer, "CTM"))
				return TRUE;
			// if( numOfBytesRead >
		   }
		}
	return FALSE;
}	// end of checkForCyberMaxx


BOOL	MaxxCom::PullPort(HWND m_hWnd)
{
 	if(ReadFromPort(SAMPLE_UPON_REQUEST) == FALSE)
	   return FALSE;
	UpdateScreen(m_hWnd);

 return TRUE;
}	// end of PullPort


BOOL	MaxxCom::SendToPort(char Command)
{
	
 return TRUE;
}	// end of SendToPort



BOOL	MaxxCom::ReadFromPort(const char* Command)
{
	if( TRUE != WriteFile( hCom, SEND_CURRENT_COORDINATE, numOfBytesToWrite, &numOfBytesRead, NULL))
		return FALSE;

	memset( inBuffer, '\0', 30 );	// Clear Buffer


 	if( bChecksum ){		// Check Sum Mode Enabled
		ReadFile(hCom, inBuffer, N2R_INCSM_ON, &numOfBytesRead ,NULL);
		iCheckSum = inBuffer[6];
	}
 	else {					// Check Sum Mode Disabled
		ReadFile(hCom, inBuffer, N2R_INCSM_OFF, &numOfBytesRead ,NULL);
 	}
	    
		yaw = (inBuffer[0] << 8) | inBuffer[1];
		yaw = (yaw * 360) / 32768;
		pitch = (inBuffer[2] << 8) | inBuffer[3];
		pitch = ((pitch * 180) / 32768) - 90;
		roll = (inBuffer[4] << 8) | inBuffer[5];
		roll = ((roll * 180) / 32768) - 90;

 	if( bChecksum )
		if( !DoCheckSum() ){
		   SetDlgItemText(GetActiveWindow(),IDC_CHECKMSG,CHECKERR);
			return FALSE;
		  }
	  SetDlgItemText(GetActiveWindow(),IDC_CHECKMSG,CHECKOK);
 return TRUE;
}	// end of ReadFromPort



void	MaxxCom::Get_YPR(int* Yaw, int* Pitch, int* Roll)
{
 *Yaw = yaw;
 *Pitch = pitch;
 *Roll = roll;
}	// end of Get_XYZ



BOOL	MaxxCom::Get_Version(HWND hWind)
{
    memset( inBuffer, '\0', 30 );

 	if( TRUE == WriteFile( hCom, REQUEST_VERSION_NUM, numOfBytesToWrite, &numOfBytesRead, NULL)){
		if(TRUE == ReadFile(hCom, inBuffer, 5, &numOfBytesRead ,NULL)){
			SetDlgItemText(hWind, IDC_VERSION, (char*)inBuffer);
				return TRUE;
		}
	}

return NULL;		  
}	// end of Get_Version


void	MaxxCom::errMsgBox(const char* errMsg)
{
  MessageBox( GetActiveWindow(), errMsg, NULL, MB_OK );
}	// end of errMsgBox	 



BOOL	MaxxCom::set9600(void)
{
	BOOL	fSuccess;

	if( iPortSpeed == 19200){
	WriteFile( hCom, TRANSITION_19200_9600, numOfBytesToWrite, &numOfBytesRead, NULL);
	
		getDCB();
		dcb.BaudRate = 9600;									
		dcb.ByteSize = 8;
		dcb.Parity = NOPARITY;
		dcb.StopBits = ONESTOPBIT;

			fSuccess = SetCommState(hCom, &dcb);
	 		if(!fSuccess){	
				errMsgBox("Init failed \n \"SetCommState\"");
	  			return FALSE;
			}
		iPortSpeed = 9600;
	}
return TRUE;
}	// end of set9600


BOOL	MaxxCom::set19200(void)
{
	BOOL	fSuccess;

	if( iPortSpeed == 9600){
		WriteFile( hCom, TRANSITION_9600_19200, numOfBytesToWrite, &numOfBytesRead, NULL);
		getDCB();
		dcb.BaudRate = 19200;									
		dcb.ByteSize = 8;
		dcb.Parity = NOPARITY;
		dcb.StopBits = ONESTOPBIT;

			fSuccess = SetCommState(hCom, &dcb);
	 		if(!fSuccess){	
				errMsgBox("Init failed \n \"SetCommState\"");
	  			return FALSE;
			}
		iPortSpeed = 19200;
	}

return TRUE;
}	// end of set19200



void	MaxxCom::UpdateScreen(HWND hWind)
{
		char printVal[30] = {NULL};
		char temp[5];

		 sprintf(printVal, "%d",yaw); 
		SetDlgItemText(hWind, IDC_X, printVal);
		 sprintf(printVal, "%d",pitch); 				   
		SetDlgItemText(hWind, IDC_Y, printVal);
		 sprintf(printVal, "%d",roll); 
		SetDlgItemText(hWind, IDC_Z, printVal);

		printVal[0]='\0';
		for(int i=0; i<6; i++){
			sprintf(temp, "%X",inBuffer[i]);
			strcat(printVal,temp);
		}
	   if( bChecksum ){
		  sprintf(temp, "%X",inBuffer[i]);
		  strcat(printVal,temp);
		}
		SetDlgItemText(hWind, IDC_OUTPUT, printVal);

}	// end of UpdateScreen



BOOL	MaxxCom::setHysteresisOn(void)
{
 return(WriteFile( hCom, HYSTERESIS_FILTER_ON, numOfBytesToWrite, &numOfBytesRead, NULL));
}	// end of setHysteresisOn



BOOL	MaxxCom::setHysteresisOff(void)
{
 return(WriteFile( hCom, HYSTERESIS_FILTER_OFF, numOfBytesToWrite, &numOfBytesRead, NULL));
}	// end of setHysteresisOff



BOOL	MaxxCom::HardReset(void)
{
  if( TRUE == WriteFile( hCom, HARD_RESET, numOfBytesToWrite, &numOfBytesRead, NULL)){
		CheckRadioButton( GetActiveWindow(), IDC_9600, IDC_19200, IDC_9600);
		return TRUE;
	}
return FALSE;
}	// end of HardReset


BOOL	MaxxCom::CheckSumOff(void)
{

	if( TRUE == WriteFile( hCom, CHECKSUM_OFF, numOfBytesToWrite, &numOfBytesRead, NULL)){
		bChecksum = FALSE;
		return TRUE;
	}
 return FALSE;
 }	// end ofCheckSumOff


BOOL	MaxxCom::CheckSumOn(void)
{
 	if( TRUE == WriteFile( hCom, CHECKSUM_ON, numOfBytesToWrite, &numOfBytesRead, NULL)){
		bChecksum = TRUE;
		return TRUE;
	}
return FALSE;
}	// end of CheckSumOff

BOOL	MaxxCom::DoCheckSum(void)
{
 int iSum;
	 iSum = (inBuffer[1] + inBuffer[3] + inBuffer[5]) & 0x000000FF;
	 if( iCheckSum == iSum)
       return TRUE;
 return FALSE;
}
