// MaxxCom.h -- header definitions for CyberMaxx Tracker
// Copyright (C) 1995  VictorMaxx Technologies, Inc.
// Author: Robert J. Gebis
// Date: 2 Oct 95 Ver 1.0

#ifndef __MAXXCOM_H__
  #define __MAXXCOM_H__

#define ABSOLUTE_MODE				"C"
#define RAW_DATA_MODE				"B"
#define ASCII_MODE					"E"
#define BINARY_MODE					"F"
#define SAMPLE_UPON_REQUEST			"G"
#define CONTINOUS_SAMPLE_MODE		"H"
#define TRANSITION_9600_19200		"K"
#define TRANSITION_19200_9600		"L"
#define CHECKSUM_ON					"M"
#define CHECKSUM_OFF				"N"
#define AVERAGE_FILTER_ON			"T"
#define AVERAGE_FILTER_OFF			"W"
#define HYSTERESIS_FILTER_ON		"P"
#define HYSTERESIS_FILTER_OFF		"O"
#define HARD_RESET					"R"
#define REQUEST_HANDSHAKE			"X"
#define REQUEST_VERSION_NUM			"V"
#define SEND_CURRENT_COORDINATE		"S"
#define SEND_CURRENT_SENSOR_DATA	"S"


#define Packet_5			0x80
#define Packet_10			0x81
#define Packet_15			0x82
#define Packet_20			0x83
#define Packet_25			0x84
#define Packet_30			0x85
#define Packet_35			0x86
#define Packet_40			0x87
#define Packet_45			0x88
#define Packet_50			0x89
#define Packet_55			0x8A
#define Packet_60			0x8B
#define Packet_65			0x8C
#define Packet_70			0x8D
#define Packet_75			0x8E
#define NL_Packet			0x8F

#define CHECKERR			"Checksum Error !!!"
#define CHECKOK				"Chacksum OK"
#define CHECKOFF			"Ckecksum Disabled"
#define HARDERR				"Hard Reset Failed !!!"
#define HARDOK				"Hard Reset Performed"
#define COM1ERR				"Can't lock to COM1 !!!"
#define COM2ERR				"Can't lock to COM2 !!!"
#define COM3ERR				"Can't lock to COM3 !!!"
#define COM4ERR				"Can't lock to COM4 !!!"
#define SELERROR			"This port is currently in use"

#define COM1_ID				1
#define COM2_ID				2
#define COM3_ID				3
#define COM4_ID				4

// number to read in check sum mode
#define N2R_INCSM_ON			7
#define N2R_INCSM_OFF			6

const char COM1[5] = "COM1";
const char COM2[5] = "COM2";
const char COM3[5] = "COM3";
const char COM4[5] = "COM4";

const int inQueue = 600;
const int outQueue = 600;
 


class MaxxCom 
	{
		public:
				MaxxCom();
				~MaxxCom();
		BOOL	OpenCom(const char*);
		BOOL	CloseCom(const HANDLE);
		BOOL	AutoDetectComPort(HWND);
		BOOL	getDCB(void);
		BOOL	checkForCyberMaxx(void);
		BOOL	set9600(void);
		BOOL	set19200(void);
		BOOL	setHysteresisOn(void);
		BOOL	setHysteresisOff(void);
		BOOL	Get_Version(HWND);
		BOOL	HardReset(void);
		BOOL	CheckSumOff(void);
		BOOL	CheckSumOn(void);
		BOOL	DoCheckSum(void);

		BOOL	SendToPort(char);
		BOOL	PullPort(HWND);
		BOOL	ReadFromPort(const char*);

		inline HANDLE	GetTempComHandle(void){return hConTemp;}
		inline BOOL		CheckIfBusy(void){ return bPortInUse;}
		inline int		GetPortId(void){return port_ID;}
		inline void		SetPortId(int id){port_ID = id;}
		inline int		GetCurrentSpeed(void){return iPortSpeed;}
		inline void		SetTempComHandle( void){hConTemp = hCom;}
		inline void		SetNewComHandle( void){hCom = hConTemp;}
		inline void		SetSpeedFlag(int speed){ iPortSpeed = speed;}

		void	Get_YPR(int*, int*, int*);		// Get Yaw,Pitch,Roll
		void	errMsgBox(const char*);	
		void	CloseCom(void);					// close any active before exit
		void	UpdateScreen(HWND);				// update static text on Main Dialog box
		 

		private:			  
			int				port_ID,
							SamplingRate,
							iCheckSum,
							iPortSpeed;		
			char			*portDiscriptor;		  // "COM1 9600 N 8 1"
			BOOL			bPortInUse,
							bChecksum;
		  	unsigned char	inBuffer[30];
		  	unsigned char	outBuffer[30];

		  	DCB				dcb;
			HANDLE			hCom,hConTemp;
			int 			yaw, pitch, roll;
			DWORD			numOfBytesToWrite,
							numToReadInBinMode,
							numOfBytesRead;

	};

#endif
