// MaxxCom.cpp -- source file for CyberMaxx Tracker
// Copyright (C) 1995  VictorMaxx Technologies, Inc.
// Author: Robert J. Gebis
// Date: 2 Oct 95 Ver 1.0


#include "stdafx.h"
#include <windows.h>
#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include "MaxxCom.h"
#include "CyberTrak.h"
#include "Cyberdlg.h"


MaxxCom::~MaxxCom() {};
			
MaxxCom::MaxxCom()
{
	SamplingRate = atoi(DEFSAMPLE);
	iPortSpeed = 19200;	// will be set in autodet mode to 9600
	bPortInUse = FALSE;
	bChecksum = FALSE;
	port_ID = 0;
	numToReadInBinMode = 6;
	numOfBytesToWrite = 1;
	memset( inBuffer, '\0', 30 );
	m_dataType = BINARY_MODE[0];
	m_TransMode = SAMPLE_UPON_REQUEST[0];
}


BOOL	MaxxCom::OpenCom(const char* com_id)
{
   	hCom = CreateFile( com_id , GENERIC_READ | GENERIC_WRITE,
								 0, NULL, OPEN_EXISTING, 0, NULL );

		if (hCom == INVALID_HANDLE_VALUE)
		   {
		      CloseCom( hCom);			//  ERROR Close ComPort
			  return FALSE;
		   }
		else
		     {	
		     	COMMTIMEOUTS timeouts;
					// Set timeouts so ReadFile does not
					// return until data is available
						timeouts.ReadIntervalTimeout = 5 ; 
						timeouts.ReadTotalTimeoutMultiplier = 5 ;
						timeouts.ReadTotalTimeoutConstant = 5 ;
						timeouts.WriteTotalTimeoutMultiplier = 0 ;
						timeouts.WriteTotalTimeoutConstant = 0 ;
						SetCommTimeouts(hCom, &timeouts ) ;

		     	// Set Comm Port to 9600 bps
			 	set9600();

			   	if( FALSE == checkForCyberMaxx())
				  {
					  CloseCom( hCom);			//  ERROR CyberMaxx not Found
					  iPortSpeed = 19200;
			  		   return FALSE;
				  }
			 }
			// flag port state
	 bPortInUse = TRUE;

 return TRUE;
}			// end of OpenCom



BOOL	MaxxCom::CloseCom(const HANDLE ComPort)
{
	WriteFile( ComPort, HARD_RESET, numOfBytesToWrite, &numOfBytesRead, NULL);
	if( TRUE == CloseHandle( ComPort))
	  {
	    bPortInUse = FALSE;
		port_ID = 0;
 		return TRUE;
	  }
	return FALSE;
}			// end of CloseCom


	
void	MaxxCom::CloseCom(void)
{
 if( port_ID == 0)		// No port connected
 	return;
	if( FALSE == CloseHandle( hCom ))
	  	errMsgBox(" Error.. Can't Close port"); 
 
}			// end of CloseCom


BOOL	MaxxCom::AutoDetectComPort(HWND wnd)
{

	if( bPortInUse == TRUE)
  	  {
  	  	errMsgBox("Device busy or not available");
		return FALSE;
  	  }	 
    

			// Try Com1
				 if(TRUE == OpenCom(COM1))
				 	{
					  CheckRadioButton(wnd , IDC_COM1, IDC_COM4, IDC_COM1);
					  port_ID = 1;
					  return TRUE;
					}
			 
			// Try Com2
				 else if(TRUE == OpenCom(COM2))
				 	{
					  CheckRadioButton( wnd , IDC_COM1, IDC_COM4, IDC_COM2);
					  port_ID = 2;
					  return TRUE;
					}

			// Try Com3
				 else if(TRUE == OpenCom(COM3))
				 	{
					  CheckRadioButton( wnd , IDC_COM1, IDC_COM4, IDC_COM3);
					  port_ID = 3;
					  return TRUE;
					}

			// Try Com4
				 else if(TRUE == OpenCom(COM4))
				 	{
					  CheckRadioButton( wnd, IDC_COM1, IDC_COM4, IDC_COM4);
					  port_ID = 4;
					  return TRUE;
					}

			// ERROR.. No CyberMaxx available
				else { errMsgBox("CyberMaxx not available\nCheck your connections"); }

 return FALSE;
}			// end AutoDetectComPort

													
 
BOOL MaxxCom::getDCB()
{
 BOOL	fSuccess;

  fSuccess = GetCommState(hCom, &dcb);

if (!fSuccess) 		  // no success?
   	return FALSE;
else return TRUE; 
}			// end of getDCB

						

BOOL	MaxxCom::checkForCyberMaxx(void)
{
  static	DWORD		numOfBytesToRead(3);
  
   SendToPort(SAMPLE_UPON_REQUEST);
   Sleep(10);
	ResetFIFO();
	memset( inBuffer, '\0', 30 );
	  if( TRUE == WriteFile( hCom,REQUEST_HANDSHAKE, numOfBytesToWrite, &numOfBytesRead, NULL)) {
		  Sleep(20);
		 if(TRUE == ReadFile(hCom, inBuffer, numOfBytesToRead, &numOfBytesRead ,NULL)) {
			if ( !strcmp((char*)inBuffer, "CTM"))
				return TRUE;
				// if( numOfBytesRead >
		   }
		}
	return FALSE;
}			// end of checkForCyberMaxx


BOOL	MaxxCom::PullPort(HWND m_hWnd)
{
 	if(ReadFromPort(TRUE) == FALSE)	// TRUE in sampled_upon_request
	   return FALSE;
	UpdateScreen(m_hWnd);

 return TRUE;
}			// end of PullPort


BOOL	MaxxCom::SendToPort(char* Command)
{
	if( TRUE != WriteFile( hCom, Command, 1, &numOfBytesRead, NULL))
		return FALSE;	
 return TRUE;
}			// end of SendToPort



BOOL	MaxxCom::ReadFromPort(BOOL TransMode)
{
 static BOOL retErr;
   if( getDataMode() == *BINARY_MODE)
	  retErr = ReadBinary(TransMode);
   else {retErr = ReadASCII(TransMode);}

 return retErr;
}			// end of ReadFromPort

BOOL	MaxxCom::ReadASCII(BOOL TransMode)
{
 static unsigned char tempBuf[30];
	memset(tempBuf, '\0', 30);
	memset( inBuffer, '\0', 30 );	// Clear Buffer

	ResetFIFO();
 if(TransMode)			// used in sampled_upon_request
	  WriteFile( hCom, SEND_CURRENT_COORDINATE, numOfBytesToWrite, &numOfBytesRead, NULL);
	Sleep(10);		// ~ (1/9600) * 8 * 20
	while( inBuffer[0] != 'Y')
		 ReadFile(hCom, inBuffer, 1, &numOfBytesRead ,NULL); // look for first control byte
	Sleep(15);
	ReadFile(hCom, inBuffer+1, 19, &numOfBytesRead ,NULL); // look for first control byte
		// Ynnn.nPsnn.nRsnn.n<cr><lr>
		//  ^ starting from first n
	strcpy((char*)tempBuf, (char*)inBuffer);	// make local copy
	tempBuf[6]	= '\0';
	tempBuf[12]= '\0';
	tempBuf[18]= '\0';
	inBuffer[18]= '\0';


	yaw = atoi((char*)tempBuf + 1);
	pitch =	atoi((char*)tempBuf + 7);
	roll = atoi((char*)tempBuf + 13);
return TRUE;
}	// end of ReadASCII()

BOOL	MaxxCom::ReadBinary(BOOL TransMode)
{
	memset( inBuffer, '\0', 30 );	// Clear Buffer

	ResetFIFO();
 	if(TransMode){			// used in sampled_upon_request
	 WriteFile( hCom, SEND_CURRENT_COORDINATE, numOfBytesToWrite, &numOfBytesRead, NULL);
	 Sleep(20);
	}
	else {					// continous mode on
		while( inBuffer[0] != 0xFF)
			 ReadFile(hCom, inBuffer, 1, &numOfBytesRead ,NULL); // look for first control byte
			 ReadFile(hCom, inBuffer + 1, 1, &numOfBytesRead ,NULL); // the secon must be 0xFF
			 Sleep(10);
	}

	if(TransMode){    		// Sample_upon_request
		if( bChecksum ){		// Check Sum Mode Enabled
			ReadFile(hCom, inBuffer, N2R_INCSM_ON, &numOfBytesRead ,NULL);
			iCheckSum = inBuffer[6];
		}
 		else ReadFile(hCom, inBuffer, N2R_INCSM_OFF, &numOfBytesRead ,NULL);	// Check Sum Mode Disabled

		yaw = (inBuffer[0] << 8) | inBuffer[1];
		yaw = (yaw * 360) >> 15;
		pitch = (inBuffer[2] << 8) | inBuffer[3];
		pitch = ((pitch * 180) >> 15) - 90;
		roll = (inBuffer[4] << 8) | inBuffer[5];
		roll = ((roll * 180) >> 15) - 90;
	}
	else {					// continous mode on
		if( bChecksum ){		// Check Sum Mode Enabled
			ReadFile(hCom, inBuffer + 2, N2R_INCSM_ON, &numOfBytesRead ,NULL);
			iCheckSum = inBuffer[8];
		}
 		else ReadFile(hCom, inBuffer + 2, N2R_INCSM_OFF, &numOfBytesRead ,NULL);	// Check Sum Mode Disabled

			numOfBytesRead += 2;	// actual data + two 0xFF bytes read above in this mode
	 		yaw = (inBuffer[2] << 8) | inBuffer[3];
			yaw = (yaw * 360) >> 15;
			pitch = (inBuffer[4] << 8) | inBuffer[5];
			pitch = ((pitch * 180) >> 15) - 90;
			roll = (inBuffer[6] << 8) | inBuffer[7];
			roll = ((roll * 180) >> 15) - 90;
	}

 	if( bChecksum )	{
		if( !DoCheckSum(TransMode) ){
		   SetDlgItemText(GetActiveWindow(),IDC_CHECKMSG,CHECKERR);
			return FALSE;
		  }
	  SetDlgItemText(GetActiveWindow(),IDC_CHECKMSG,CHECKOK);
	}
 return TRUE;
}			// end of ReadBinary()

void	MaxxCom::Get_YPR(int* Yaw, int* Pitch, int* Roll)
{
 *Yaw = yaw;
 *Pitch = pitch;
 *Roll = roll;
}			// end of Get_XYZ



BOOL	MaxxCom::Get_Version(HWND hWind)
{
    memset( inBuffer, '\0', 30 );

 	if( TRUE == WriteFile( hCom, REQUEST_VERSION_NUM, numOfBytesToWrite, &numOfBytesRead, NULL)){
		if(TRUE == ReadFile(hCom, inBuffer, 5, &numOfBytesRead ,NULL)){
			SetDlgItemText(hWind, IDC_VERSION, (char*)inBuffer);
				return TRUE;
		}
	}

return FALSE;		  
}			// end of Get_Version


void	MaxxCom::errMsgBox(const char* errMsg)
{
  MessageBox( GetActiveWindow(), errMsg, NULL, MB_OK | MB_ICONEXCLAMATION );
}			// end of errMsgBox	 



BOOL	MaxxCom::set9600(void)
{
	BOOL	fSuccess;

	if( iPortSpeed == 19200){
	WriteFile( hCom, TRANSITION_19200_9600, numOfBytesToWrite, &numOfBytesRead, NULL);
	
		getDCB();
		dcb.BaudRate = 9600;									
		dcb.ByteSize = 8;
		dcb.Parity = NOPARITY;
		dcb.StopBits = ONESTOPBIT;

			fSuccess = SetCommState(hCom, &dcb);
	 		if(!fSuccess){	
				errMsgBox("Init failed \n \"SetCommState\"");
	  			return FALSE;
			}
		iPortSpeed = 9600;
	}
return TRUE;
}			// end of set9600


BOOL	MaxxCom::set19200(void)
{
	BOOL	fSuccess;

	if( iPortSpeed == 9600){
		WriteFile( hCom, TRANSITION_9600_19200, numOfBytesToWrite, &numOfBytesRead, NULL);
		getDCB();
		dcb.BaudRate = 19200;									
		dcb.ByteSize = 8;
		dcb.Parity = NOPARITY;
		dcb.StopBits = ONESTOPBIT;

			fSuccess = SetCommState(hCom, &dcb);
	 		if(!fSuccess){	
				errMsgBox("Init failed \n \"SetCommState\"");
	  			return FALSE;
			}
		iPortSpeed = 19200;
	}

return TRUE;
}			// end of set19200



void	MaxxCom::UpdateScreen(HWND hWind)
{
		static char printVal[30] = {'\0'};
		char temp[5];

		 sprintf(printVal, "%d",yaw); 
		SetDlgItemText(hWind, IDC_Y, printVal);
		 sprintf(printVal, "%d",pitch); 				   
		SetDlgItemText(hWind, IDC_X, printVal);
		 sprintf(printVal, "%d",roll); 
		SetDlgItemText(hWind, IDC_Z, printVal);

		memset(printVal, '\0', 30);
		if(getDataMode() == *BINARY_MODE){
			for(int i=0; i< (int)numOfBytesRead; i++){
				sprintf(temp, "%X",inBuffer[i]);
			//	printVal[i] = (char)*temp;
				strcat(printVal,temp);
			}
		}
		else {	// ASCII mode
		  strcpy(printVal,(char*)inBuffer); 
		}
		SetDlgItemText(hWind, IDC_OUTPUT, printVal);

}			// end of UpdateScreen



BOOL	MaxxCom::setHysteresisOn(void)
{
 return(WriteFile( hCom, HYSTERESIS_FILTER_ON, numOfBytesToWrite, &numOfBytesRead, NULL));
}			// end of setHysteresisOn



BOOL	MaxxCom::setHysteresisOff(void)
{
 return(WriteFile( hCom, HYSTERESIS_FILTER_OFF, numOfBytesToWrite, &numOfBytesRead, NULL));
}			// end of setHysteresisOff



BOOL	MaxxCom::HardReset(void)
{
  if( TRUE == WriteFile( hCom, HARD_RESET, numOfBytesToWrite, &numOfBytesRead, NULL)){
	  set9600();
		  WriteFile( hCom, HARD_RESET, numOfBytesToWrite, &numOfBytesRead, NULL); // just to make sure
		  CheckRadioButton( GetActiveWindow(), IDC_ASCII, IDC_BINARY, IDC_BINARY);
		  CheckRadioButton( GetActiveWindow(), IDC_CHECK_ON, IDC_CHECK_OFF, IDC_CHECK_OFF);
		  CheckRadioButton( GetActiveWindow(), IDC_UPON_REQUEST, IDC_CONTINOUS, IDC_UPON_REQUEST);
		  CheckRadioButton( GetActiveWindow(), IDC_AVR_FIL_ON, IDC_AVR_FIL_OFF, IDC_AVR_FIL_ON);
		  CheckRadioButton( GetActiveWindow(), IDC_HYSTER_ON, IDC_HYSTER_OFF, IDC_HYSTER_OFF);
		  CheckRadioButton( GetActiveWindow(), IDC_9600, IDC_19200, IDC_9600);

		bChecksum = FALSE;
		return TRUE;
	}
return FALSE;
}			// end of HardReset


BOOL	MaxxCom::CheckSumOff(void)
{

	if( TRUE == WriteFile( hCom, CHECKSUM_OFF, numOfBytesToWrite, &numOfBytesRead, NULL)){
		bChecksum = FALSE;
		return TRUE;
	}
 bChecksum = TRUE;
 return FALSE;
 }			// end ofCheckSumOff


BOOL	MaxxCom::CheckSumOn(void)
{
 	if( TRUE == WriteFile( hCom, CHECKSUM_ON, numOfBytesToWrite, &numOfBytesRead, NULL)){
		bChecksum = TRUE;
		return TRUE;
	}
bChecksum = FALSE;
return FALSE;
}			// end of CheckSumOff

BOOL	MaxxCom::DoCheckSum(BOOL TransMode)
{
 int iSum;
	if(TransMode)	// Sample Upon request
	 iSum = (inBuffer[1]+inBuffer[3]+inBuffer[5]) & 0xFF;
	else iSum = (inBuffer[3]+inBuffer[5]+inBuffer[7]) & 0xFF;
	 if( iCheckSum == iSum)
       return TRUE;
 return FALSE;
}

void	MaxxCom::SetSamplingRate(int rate)
{
 unsigned char cRate[2] = {0x80};

 SamplingRate = rate;
 rate = (rate/5)-1;
 cRate[0] += (unsigned char)rate;
  WriteFile( hCom, cRate, 1, &numOfBytesRead, NULL);
}			// end of SetSamplingRate()

void	MaxxCom::ResetFIFO(void)
{
 	PurgeComm(hCom,PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);
}