// MaxxCom.h -- header definitions for CyberMaxx Tracker
// Copyright (C) 1995  VictorMaxx Technologies, Inc.
// Author: Robert J. Gebis
// Date: 2 Oct 95 Ver 1.0
// Date: 19 Jan 96
//		Sampling rate updated, menual comm port select fixed
//		Proper msg displayed, version readout corrected,
//		additional delay added in checkForCyberMaxx to prevent from
//		incoplite packet 
// Date: 23 Jan 96
//		ASCII transfer mode addad, checksum bug fixed

#ifndef __MAXXCOM_H__
  #define __MAXXCOM_H__

#define ABSOLUTE_MODE				"C"
#define RAW_DATA_MODE				"B"
#define ASCII_MODE					"E"
#define BINARY_MODE					"F"
#define SAMPLE_UPON_REQUEST			"G"
#define CONTINOUS_SAMPLE_MODE		"H"
#define TRANSITION_9600_19200		"K"
#define TRANSITION_19200_9600		"L"
#define CHECKSUM_ON					"M"
#define CHECKSUM_OFF				"N"
#define AVERAGE_FILTER_ON			"T"
#define AVERAGE_FILTER_OFF			"W"
#define HYSTERESIS_FILTER_ON		"P"
#define HYSTERESIS_FILTER_OFF		"O"
#define HARD_RESET					"R"
#define REQUEST_HANDSHAKE			"X"
#define REQUEST_VERSION_NUM			"V"
#define SEND_CURRENT_COORDINATE		"S"
#define SEND_CURRENT_SENSOR_DATA	"S"

#define MAXPACK				75			// higher sampling mode
#define MINPACK				5			// lower sampling mode
#define DEFSAMPLE			"40"		// starting sampling speed

#define CHECKERR			"Checksum Error !!!"
#define CHECKOK				"Checksum OK"
#define CHECKOFF			"Checksum Disabled"
#define CHECKENB			"Checksum Enabled"
#define AVFILTEROFF			"Averaging Filter Disabled"
#define AVFILTERENB			"Averaging Filter Enabled"
#define COM9600				"Serial port set to 9600"
#define COM19200			"Serial port set to 19200"
#define HYSTEROFF			"Hysteresis Filter Disabled"
#define HYSTERON			"Hysteresis Filter Enabled"
#define HARDERR				"Hard Reset Failed !!!"
#define HARDOK				"Hard Reset Performed"
#define COM1ERR				"Can't lock to COM1 !!!"
#define COM2ERR				"Can't lock to COM2 !!!"
#define COM3ERR				"Can't lock to COM3 !!!"
#define COM4ERR				"Can't lock to COM4 !!!"
#define MAXFOUND			"CyberMaxx found !!!"
#define SELERROR			"This port is currently in use"

#define COM1_ID				1
#define COM2_ID				2
#define COM3_ID				3
#define COM4_ID				4

// number to read in check sum mode
#define N2R_INCSM_ON		7
#define N2R_INCSM_OFF		6

const char COM1[5] = "COM1";
const char COM2[5] = "COM2";
const char COM3[5] = "COM3";
const char COM4[5] = "COM4";


class MaxxCom {

	public:
				MaxxCom();
				~MaxxCom();
		BOOL	OpenCom(const char*);			// 
		BOOL	CloseCom(const HANDLE);
		BOOL	AutoDetectComPort(HWND);
		BOOL	getDCB(void);
		BOOL	checkForCyberMaxx(void);
		BOOL	set9600(void);
		BOOL	set19200(void);
		BOOL	setHysteresisOn(void);
		BOOL	setHysteresisOff(void);
		BOOL	Get_Version(HWND);
		BOOL	HardReset(void);
		BOOL	CheckSumOff(void);
		BOOL	CheckSumOn(void);
		BOOL	DoCheckSum(BOOL);

		BOOL	SendToPort(char*);
		BOOL	PullPort(HWND);
		BOOL	ReadFromPort(BOOL);
		BOOL	ReadASCII(BOOL);
		BOOL	ReadBinary(BOOL);

		inline HANDLE	GetTempComHandle(void){return hConTemp;}
		inline HANDLE	GetComHandle(void){return hCom;}
		inline BOOL		CheckIfBusy(void){ return bPortInUse;}
		inline int		GetPortId(void){return port_ID;}
		inline void		SetPortId(int id){port_ID = id;}
		inline int		GetCurrentSpeed(void){return iPortSpeed;}
		inline void		SetTempComHandle( void){hConTemp = hCom;}
		inline void		SetNewComHandle( void){hCom = hConTemp;}
		inline void		SetSpeedFlag(int speed){ iPortSpeed = speed;}
		inline char		getDataMode(void){ return m_dataType;}
		inline void		setDataMode(char tp){ m_dataType = tp;}
		inline char		getTransMode(void){ return m_TransMode;}
		inline void		setTransMode(char tp){ m_TransMode = tp;}
		inline int		getSamplingRate(void){return SamplingRate;}
		
		void			SetSamplingRate(int rate);
		void			Get_YPR(int*, int*, int*);		// Get Yaw,Pitch,Roll
		void			errMsgBox(const char*);	
		void			CloseCom(void);					// close any active before exit
		void			UpdateScreen(HWND);				// update static text on Main Dialog box
		void			ResetFIFO(void);

	private:			  
			int				port_ID,
							SamplingRate,
							iCheckSum,
							iPortSpeed;		
			char			m_dataType;				// used to flag binary or ascii mode
			char			m_TransMode;
			BOOL			bPortInUse,
							bChecksum;
		  	unsigned char	inBuffer[30];
		  	//unsigned char	outBuffer[30];

		  	DCB				dcb;
			HANDLE			hCom,hConTemp;
			int 			yaw, pitch, roll;
			DWORD			numOfBytesToWrite,
							numToReadInBinMode,
							numOfBytesRead;

	};

#endif		// nothing else after this line
